"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from gr4vy.types import UnrecognizedStr
from typing import Literal, Union


CVVResponseCode = Union[
    Literal[
        "match",
        "no_match",
        "unavailable",
        "not_provided",
    ],
    UnrecognizedStr,
]



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from gr4vy.types import BaseModel, Nullable, OptionalNullable, UNSET, UNSET_SENTINEL
from gr4vy.utils import (
    FieldMetadata,
    HeaderMetadata,
    PathParamMetadata,
    QueryParamMetadata,
)
import pydantic
from pydantic import model_serializer
from typing import Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class ListTransactionEventsGlobalsTypedDict(TypedDict):
    merchant_account_id: NotRequired[str]
    r"""The ID of the merchant account to use for this request."""


class ListTransactionEventsGlobals(BaseModel):
    merchant_account_id: Annotated[
        Optional[str],
        pydantic.Field(alias="x-gr4vy-merchant-account-id"),
        FieldMetadata(header=HeaderMetadata(style="simple", explode=False)),
    ] = None
    r"""The ID of the merchant account to use for this request."""


class ListTransactionEventsRequestTypedDict(TypedDict):
    transaction_id: str
    r"""The ID of the transaction"""
    cursor: NotRequired[Nullable[str]]
    r"""A pointer to the page of results to return."""
    limit: NotRequired[int]
    r"""The maximum number of items that are at returned."""
    merchant_account_id: NotRequired[str]
    r"""The ID of the merchant account to use for this request."""


class ListTransactionEventsRequest(BaseModel):
    transaction_id: Annotated[
        str, FieldMetadata(path=PathParamMetadata(style="simple", explode=False))
    ]
    r"""The ID of the transaction"""

    cursor: Annotated[
        OptionalNullable[str],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET
    r"""A pointer to the page of results to return."""

    limit: Annotated[
        Optional[int],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = 100
    r"""The maximum number of items that are at returned."""

    merchant_account_id: Annotated[
        Optional[str],
        pydantic.Field(alias="x-gr4vy-merchant-account-id"),
        FieldMetadata(header=HeaderMetadata(style="simple", explode=False)),
    ] = None
    r"""The ID of the merchant account to use for this request."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["cursor", "limit", "merchant_account_id"]
        nullable_fields = ["cursor"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from gr4vy.types import (
    BaseModel,
    Nullable,
    OptionalNullable,
    UNSET,
    UNSET_SENTINEL,
    UnrecognizedStr,
)
from gr4vy.utils import validate_open_enum
from pydantic import model_serializer
from pydantic.functional_validators import PlainValidator
from typing import Literal, Union
from typing_extensions import Annotated, NotRequired, TypedDict


Type = Union[
    Literal[
        "TANGIBLE",
        "NON_TANGIBLE",
    ],
    UnrecognizedStr,
]


class ForterAntiFraudOptionsCartItemBasicItemDataTypedDict(TypedDict):
    type: NotRequired[Nullable[Type]]
    r"""Indicates whether the item is a physical good or a service/digital item."""


class ForterAntiFraudOptionsCartItemBasicItemData(BaseModel):
    type: Annotated[
        OptionalNullable[Type], PlainValidator(validate_open_enum(False))
    ] = UNSET
    r"""Indicates whether the item is a physical good or a service/digital item."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["type"]
        nullable_fields = ["type"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from gr4vy.types import UnrecognizedStr
from typing import Literal, Union


AdyenCardAutoRescueScenariosEnum = Union[
    Literal[
        "AutoRescueSuccessfulFirst",
        "AutoRescueSuccessfulSecond",
        "AutoRescueFailed",
        "AutoRescueFraud",
    ],
    UnrecognizedStr,
]



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from gr4vy.types import BaseModel
from typing_extensions import TypedDict


class GooglePaySessionTypedDict(TypedDict):
    gateway_merchant_id: str
    r"""The gateway ID for the merchant as assigned by our platform."""
    token: str
    r"""The session token for Google Pay."""


class GooglePaySession(BaseModel):
    gateway_merchant_id: str
    r"""The gateway ID for the merchant as assigned by our platform."""

    token: str
    r"""The session token for Google Pay."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from .basesdk import BaseSDK
from gr4vy import errors, models, utils
from gr4vy._hooks import HookContext
from gr4vy.types import OptionalNullable, UNSET
from gr4vy.utils import get_security_from_env
from gr4vy.utils.unmarshal_json_response import unmarshal_json_response
from jsonpath import JSONPath
from typing import Any, Dict, List, Mapping, Optional, Union


class MerchantAccountsSDK(BaseSDK):
    def list(
        self,
        *,
        cursor: OptionalNullable[str] = UNSET,
        limit: Optional[int] = 20,
        search: OptionalNullable[str] = UNSET,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> Optional[models.ListMerchantAccountsResponse]:
        r"""List all merchant accounts

        List all merchant accounts in an instance.

        :param cursor: A pointer to the page of results to return.
        :param limit: The maximum number of items that are at returned.
        :param search: The search term to filter merchant accounts by.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.ListMerchantAccountsRequest(
            cursor=cursor,
            limit=limit,
            search=search,
        )

        req = self._build_request(
            method="GET",
            path="/merchant-accounts",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config
            else:
                retries = utils.RetryConfig(
                    "backoff", utils.BackoffStrategy(200, 200, 1, 1000), True
                )

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["5XX"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="list_merchant_accounts",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=[
                "400",
                "401",
                "403",
                "404",
                "405",
                "409",
                "422",
                "425",
                "429",
                "4XX",
                "500",
                "502",
                "504",
                "5XX",
            ],
            retry_config=retry_config,
        )

        def next_func() -> Optional[models.ListMerchantAccountsResponse]:
            body = utils.unmarshal_json(http_res.text, Union[Dict[Any, Any], List[Any]])
            next_cursor = JSONPath("$.next_cursor").parse(body)

            if len(next_cursor) == 0:
                return None

            next_cursor = next_cursor[0]
            if next_cursor is None or str(next_cursor).strip() == "":
                return None

            return self.list(
                cursor=next_cursor,
                limit=limit,
                search=search,
                retries=retries,
            )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return models.ListMerchantAccountsResponse(
                result=unmarshal_json_response(models.MerchantAccounts, http_res),
                next=next_func,
            )
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(errors.Error400Data, http_res)
            raise errors.Error400(response_data, http_res)
        if utils.match_response(http_res, "401", "application/json"):
            response_data = unmarshal_json_response(errors.Error401Data, http_res)
            raise errors.Error401(response_data, http_res)
        if utils.match_response(http_res, "403", "application/json"):
            response_data = unmarshal_json_response(errors.Error403Data, http_res)
            raise errors.Error403(response_data, http_res)
        if utils.match_response(http_res, "404", "application/json"):
            response_data = unmarshal_json_response(errors.Error404Data, http_res)
            raise errors.Error404(response_data, http_res)
        if utils.match_response(http_res, "405", "application/json"):
            response_data = unmarshal_json_response(errors.Error405Data, http_res)
            raise errors.Error405(response_data, http_res)
        if utils.match_response(http_res, "409", "application/json"):
            response_data = unmarshal_json_response(errors.Error409Data, http_res)
            raise errors.Error409(response_data, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                errors.HTTPValidationErrorData, http_res
            )
            raise errors.HTTPValidationError(response_data, http_res)
        if utils.match_response(http_res, "425", "application/json"):
            response_data = unmarshal_json_response(errors.Error425Data, http_res)
            raise errors.Error425(response_data, http_res)
        if utils.match_response(http_res, "429", "application/json"):
            response_data = unmarshal_json_response(errors.Error429Data, http_res)
            raise errors.Error429(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(errors.Error500Data, http_res)
            raise errors.Error500(response_data, http_res)
        if utils.match_response(http_res, "502", "application/json"):
            response_data = unmarshal_json_response(errors.Error502Data, http_res)
            raise errors.Error502(response_data, http_res)
        if utils.match_response(http_res, "504", "application/json"):
            response_data = unmarshal_json_response(errors.Error504Data, http_res)
            raise errors.Error504(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)

        raise errors.APIError("Unexpected response received", http_res)

    async def list_async(
        self,
        *,
        cursor: OptionalNullable[str] = UNSET,
        limit: Optional[int] = 20,
        search: OptionalNullable[str] = UNSET,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> Optional[models.ListMerchantAccountsResponse]:
        r"""List all merchant accounts

        List all merchant accounts in an instance.

        :param cursor: A pointer to the page of results to return.
        :param limit: The maximum number of items that are at returned.
        :param search: The search term to filter merchant accounts by.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.ListMerchantAccountsRequest(
            cursor=cursor,
            limit=limit,
            search=search,
        )

        req = self._build_request_async(
            method="GET",
            path="/merchant-accounts",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config
            else:
                retries = utils.RetryConfig(
                    "backoff", utils.BackoffStrategy(200, 200, 1, 1000), True
                )

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["5XX"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="list_merchant_accounts",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=[
                "400",
                "401",
                "403",
                "404",
                "405",
                "409",
                "422",
                "425",
                "429",
                "4XX",
                "500",
                "502",
                "504",
                "5XX",
            ],
            retry_config=retry_config,
        )

        def next_func() -> Optional[models.ListMerchantAccountsResponse]:
            body = utils.unmarshal_json(http_res.text, Union[Dict[Any, Any], List[Any]])
            next_cursor = JSONPath("$.next_cursor").parse(body)

            if len(next_cursor) == 0:
                return None

            next_cursor = next_cursor[0]
            if next_cursor is None or str(next_cursor).strip() == "":
                return None

            return self.list(
                cursor=next_cursor,
                limit=limit,
                search=search,
                retries=retries,
            )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return models.ListMerchantAccountsResponse(
                result=unmarshal_json_response(models.MerchantAccounts, http_res),
                next=next_func,
            )
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(errors.Error400Data, http_res)
            raise errors.Error400(response_data, http_res)
        if utils.match_response(http_res, "401", "application/json"):
            response_data = unmarshal_json_response(errors.Error401Data, http_res)
            raise errors.Error401(response_data, http_res)
        if utils.match_response(http_res, "403", "application/json"):
            response_data = unmarshal_json_response(errors.Error403Data, http_res)
            raise errors.Error403(response_data, http_res)
        if utils.match_response(http_res, "404", "application/json"):
            response_data = unmarshal_json_response(errors.Error404Data, http_res)
            raise errors.Error404(response_data, http_res)
        if utils.match_response(http_res, "405", "application/json"):
            response_data = unmarshal_json_response(errors.Error405Data, http_res)
            raise errors.Error405(response_data, http_res)
        if utils.match_response(http_res, "409", "application/json"):
            response_data = unmarshal_json_response(errors.Error409Data, http_res)
            raise errors.Error409(response_data, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                errors.HTTPValidationErrorData, http_res
            )
            raise errors.HTTPValidationError(response_data, http_res)
        if utils.match_response(http_res, "425", "application/json"):
            response_data = unmarshal_json_response(errors.Error425Data, http_res)
            raise errors.Error425(response_data, http_res)
        if utils.match_response(http_res, "429", "application/json"):
            response_data = unmarshal_json_response(errors.Error429Data, http_res)
            raise errors.Error429(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(errors.Error500Data, http_res)
            raise errors.Error500(response_data, http_res)
        if utils.match_response(http_res, "502", "application/json"):
            response_data = unmarshal_json_response(errors.Error502Data, http_res)
            raise errors.Error502(response_data, http_res)
        if utils.match_response(http_res, "504", "application/json"):
            response_data = unmarshal_json_response(errors.Error504Data, http_res)
            raise errors.Error504(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)

        raise errors.APIError("Unexpected response received", http_res)

    def create(
        self,
        *,
        id: str,
        display_name: str,
        account_updater_enabled: Optional[bool] = False,
        account_updater_request_encryption_key: OptionalNullable[str] = UNSET,
        account_updater_request_encryption_key_id: OptionalNullable[str] = UNSET,
        account_updater_response_decryption_key: OptionalNullable[str] = UNSET,
        account_updater_response_decryption_key_id: OptionalNullable[str] = UNSET,
        over_capture_amount: OptionalNullable[int] = UNSET,
        over_capture_percentage: OptionalNullable[int] = UNSET,
        loon_client_key: OptionalNullable[str] = UNSET,
        loon_secret_key: OptionalNullable[str] = UNSET,
        loon_accepted_schemes: OptionalNullable[List[models.CardScheme]] = UNSET,
        visa_network_tokens_requestor_id: OptionalNullable[str] = UNSET,
        visa_network_tokens_app_id: OptionalNullable[str] = UNSET,
        amex_network_tokens_requestor_id: OptionalNullable[str] = UNSET,
        amex_network_tokens_app_id: OptionalNullable[str] = UNSET,
        mastercard_network_tokens_requestor_id: OptionalNullable[str] = UNSET,
        mastercard_network_tokens_app_id: OptionalNullable[str] = UNSET,
        async_network_tokens_enabled: Optional[bool] = False,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.MerchantAccount:
        r"""Create a merchant account

        Create a new merchant account in an instance.

        :param id: The ID for the merchant account.
        :param display_name: The display name for the merchant account.
        :param account_updater_enabled: Whether the Real-Time Account Updater service is enabled for this merchant account. The Account Updater service is used to update card details when cards are lost, stolen or expired. If the field is not set or if it's set to `false`, the Account Updater service doesn't get called if a payment fails with expired or invalid card details. If the field is set to `true`, the service is called. Please note that for this to work the other `account_updater_* fields` must be set as well.
        :param account_updater_request_encryption_key: The public key used to encrypt the request to the Real-Time Account Updater service. The Account Updater service is used to update card details when cards are lost, stolen or expired. If the field is not set or if it's set to `null`, the Account Updater service doesn't get called. If the field is set, the other `account_updater_*` fields must be set as well.
        :param account_updater_request_encryption_key_id: The ID of the key used to encrypt the request to the Real-Time Account Updater service. The Account Updater service is used to update card details when cards are lost, stolen or expired. If the field is not set or if it's set to `null`, the Account Updater service doesn't get called. If the field is set, the other `account_updater_*` fields must be set as well.
        :param account_updater_response_decryption_key: The key used to decrypt the response from the Real-Time Account Updater service. The Account Updater service is used to update card details when cards are lost, stolen or expired. If the field is not set or if it's set to `null`, the Account Updater service doesn't get called. If the field is set, the other `account_updater_*` fields must be set as well.
        :param account_updater_response_decryption_key_id: The ID of the key used to decrypt the request from the Real-Time Account Updater service. The Account Updater service is used to update card details when cards are lost, stolen or expired. If the field is not set or if it's set to `null`, the Account Updater service doesn't get called. If the field is set, the other `account_updater_*` fields must be set as well.
        :param over_capture_amount: The maximum monetary amount allowed for over-capture, in the smallest currency unit, for example `1299` cents to allow for an over-capture of `$12.99`.
        :param over_capture_percentage: The maximum percentage allowed for over-capture, for example `25` to allow for an over-capture of `25%` of the original transaction amount.
        :param loon_client_key: Client key provided by Pagos to authenticate to the Loon API. Loon is the Account Updater service we use and if the field is not set or if it's set to null, the Account Updater service doesn't get configured. If the field is set to `null`, the other `loon_*` fields must be set to null as well.
        :param loon_secret_key: Secret key provided by Pagos to authenticate to the Loon API. Loon is the Account Updater service we use and if the field is not set or if it's set to null, the Account Updater service doesn't get configured. If the field is set to `null`, the other `loon_*` fields must be set to null as well.
        :param loon_accepted_schemes: Card schemes accepted when creating jobs using this set of Loon API keys. Loon is the Account Updater service we use and if the field is not set or if it's set to null, the Account Updater service doesn't get configured. If the field is set to `null`, the other `loon_*` fields must be set to null as well.
        :param visa_network_tokens_requestor_id: Requestor ID provided for Visa after onboarding to use Network Tokens.
        :param visa_network_tokens_app_id: Application ID provided for Visa after onboarding to use Network Tokens.
        :param amex_network_tokens_requestor_id: Requestor ID provided for American Express after onboarding to use Network Tokens.
        :param amex_network_tokens_app_id: Application ID provided for American Express after onboarding to use Network Tokens.
        :param mastercard_network_tokens_requestor_id: Requestor ID provided for Mastercard after onboarding to use Network Tokens.
        :param mastercard_network_tokens_app_id: Application ID provided for Mastercard after onboarding to use Network Tokens.
        :param async_network_tokens_enabled: When enabled network tokens will be generated asynchronously and only used on subsequent transactions to speed up transaction processing.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.MerchantAccountCreate(
            account_updater_enabled=account_updater_enabled,
            account_updater_request_encryption_key=account_updater_request_encryption_key,
            account_updater_request_encryption_key_id=account_updater_request_encryption_key_id,
            account_updater_response_decryption_key=account_updater_response_decryption_key,
            account_updater_response_decryption_key_id=account_updater_response_decryption_key_id,
            over_capture_amount=over_capture_amount,
            over_capture_percentage=over_capture_percentage,
            loon_client_key=loon_client_key,
            loon_secret_key=loon_secret_key,
            loon_accepted_schemes=loon_accepted_schemes,
            visa_network_tokens_requestor_id=visa_network_tokens_requestor_id,
            visa_network_tokens_app_id=visa_network_tokens_app_id,
            amex_network_tokens_requestor_id=amex_network_tokens_requestor_id,
            amex_network_tokens_app_id=amex_network_tokens_app_id,
            mastercard_network_tokens_requestor_id=mastercard_network_tokens_requestor_id,
            mastercard_network_tokens_app_id=mastercard_network_tokens_app_id,
            async_network_tokens_enabled=async_network_tokens_enabled,
            id=id,
            display_name=display_name,
        )

        req = self._build_request(
            method="POST",
            path="/merchant-accounts",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request, False, False, "json", models.MerchantAccountCreate
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="create_merchant_account",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=[
                "400",
                "401",
                "403",
                "404",
                "405",
                "409",
                "422",
                "425",
                "429",
                "4XX",
                "500",
                "502",
                "504",
                "5XX",
            ],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "201", "application/json"):
            return unmarshal_json_response(models.MerchantAccount, http_res)
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(errors.Error400Data, http_res)
            raise errors.Error400(response_data, http_res)
        if utils.match_response(http_res, "401", "application/json"):
            response_data = unmarshal_json_response(errors.Error401Data, http_res)
            raise errors.Error401(response_data, http_res)
        if utils.match_response(http_res, "403", "application/json"):
            response_data = unmarshal_json_response(errors.Error403Data, http_res)
            raise errors.Error403(response_data, http_res)
        if utils.match_response(http_res, "404", "application/json"):
            response_data = unmarshal_json_response(errors.Error404Data, http_res)
            raise errors.Error404(response_data, http_res)
        if utils.match_response(http_res, "405", "application/json"):
            response_data = unmarshal_json_response(errors.Error405Data, http_res)
            raise errors.Error405(response_data, http_res)
        if utils.match_response(http_res, "409", "application/json"):
            response_data = unmarshal_json_response(errors.Error409Data, http_res)
            raise errors.Error409(response_data, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                errors.HTTPValidationErrorData, http_res
            )
            raise errors.HTTPValidationError(response_data, http_res)
        if utils.match_response(http_res, "425", "application/json"):
            response_data = unmarshal_json_response(errors.Error425Data, http_res)
            raise errors.Error425(response_data, http_res)
        if utils.match_response(http_res, "429", "application/json"):
            response_data = unmarshal_json_response(errors.Error429Data, http_res)
            raise errors.Error429(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(errors.Error500Data, http_res)
            raise errors.Error500(response_data, http_res)
        if utils.match_response(http_res, "502", "application/json"):
            response_data = unmarshal_json_response(errors.Error502Data, http_res)
            raise errors.Error502(response_data, http_res)
        if utils.match_response(http_res, "504", "application/json"):
            response_data = unmarshal_json_response(errors.Error504Data, http_res)
            raise errors.Error504(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)

        raise errors.APIError("Unexpected response received", http_res)

    async def create_async(
        self,
        *,
        id: str,
        display_name: str,
        account_updater_enabled: Optional[bool] = False,
        account_updater_request_encryption_key: OptionalNullable[str] = UNSET,
        account_updater_request_encryption_key_id: OptionalNullable[str] = UNSET,
        account_updater_response_decryption_key: OptionalNullable[str] = UNSET,
        account_updater_response_decryption_key_id: OptionalNullable[str] = UNSET,
        over_capture_amount: OptionalNullable[int] = UNSET,
        over_capture_percentage: OptionalNullable[int] = UNSET,
        loon_client_key: OptionalNullable[str] = UNSET,
        loon_secret_key: OptionalNullable[str] = UNSET,
        loon_accepted_schemes: OptionalNullable[List[models.CardScheme]] = UNSET,
        visa_network_tokens_requestor_id: OptionalNullable[str] = UNSET,
        visa_network_tokens_app_id: OptionalNullable[str] = UNSET,
        amex_network_tokens_requestor_id: OptionalNullable[str] = UNSET,
        amex_network_tokens_app_id: OptionalNullable[str] = UNSET,
        mastercard_network_tokens_requestor_id: OptionalNullable[str] = UNSET,
        mastercard_network_tokens_app_id: OptionalNullable[str] = UNSET,
        async_network_tokens_enabled: Optional[bool] = False,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.MerchantAccount:
        r"""Create a merchant account

        Create a new merchant account in an instance.

        :param id: The ID for the merchant account.
        :param display_name: The display name for the merchant account.
        :param account_updater_enabled: Whether the Real-Time Account Updater service is enabled for this merchant account. The Account Updater service is used to update card details when cards are lost, stolen or expired. If the field is not set or if it's set to `false`, the Account Updater service doesn't get called if a payment fails with expired or invalid card details. If the field is set to `true`, the service is called. Please note that for this to work the other `account_updater_* fields` must be set as well.
        :param account_updater_request_encryption_key: The public key used to encrypt the request to the Real-Time Account Updater service. The Account Updater service is used to update card details when cards are lost, stolen or expired. If the field is not set or if it's set to `null`, the Account Updater service doesn't get called. If the field is set, the other `account_updater_*` fields must be set as well.
        :param account_updater_request_encryption_key_id: The ID of the key used to encrypt the request to the Real-Time Account Updater service. The Account Updater service is used to update card details when cards are lost, stolen or expired. If the field is not set or if it's set to `null`, the Account Updater service doesn't get called. If the field is set, the other `account_updater_*` fields must be set as well.
        :param account_updater_response_decryption_key: The key used to decrypt the response from the Real-Time Account Updater service. The Account Updater service is used to update card details when cards are lost, stolen or expired. If the field is not set or if it's set to `null`, the Account Updater service doesn't get called. If the field is set, the other `account_updater_*` fields must be set as well.
        :param account_updater_response_decryption_key_id: The ID of the key used to decrypt the request from the Real-Time Account Updater service. The Account Updater service is used to update card details when cards are lost, stolen or expired. If the field is not set or if it's set to `null`, the Account Updater service doesn't get called. If the field is set, the other `account_updater_*` fields must be set as well.
        :param over_capture_amount: The maximum monetary amount allowed for over-capture, in the smallest currency unit, for example `1299` cents to allow for an over-capture of `$12.99`.
        :param over_capture_percentage: The maximum percentage allowed for over-capture, for example `25` to allow for an over-capture of `25%` of the original transaction amount.
        :param loon_client_key: Client key provided by Pagos to authenticate to the Loon API. Loon is the Account Updater service we use and if the field is not set or if it's set to null, the Account Updater service doesn't get configured. If the field is set to `null`, the other `loon_*` fields must be set to null as well.
        :param loon_secret_key: Secret key provided by Pagos to authenticate to the Loon API. Loon is the Account Updater service we use and if the field is not set or if it's set to null, the Account Updater service doesn't get configured. If the field is set to `null`, the other `loon_*` fields must be set to null as well.
        :param loon_accepted_schemes: Card schemes accepted when creating jobs using this set of Loon API keys. Loon is the Account Updater service we use and if the field is not set or if it's set to null, the Account Updater service doesn't get configured. If the field is set to `null`, the other `loon_*` fields must be set to null as well.
        :param visa_network_tokens_requestor_id: Requestor ID provided for Visa after onboarding to use Network Tokens.
        :param visa_network_tokens_app_id: Application ID provided for Visa after onboarding to use Network Tokens.
        :param amex_network_tokens_requestor_id: Requestor ID provided for American Express after onboarding to use Network Tokens.
        :param amex_network_tokens_app_id: Application ID provided for American Express after onboarding to use Network Tokens.
        :param mastercard_network_tokens_requestor_id: Requestor ID provided for Mastercard after onboarding to use Network Tokens.
        :param mastercard_network_tokens_app_id: Application ID provided for Mastercard after onboarding to use Network Tokens.
        :param async_network_tokens_enabled: When enabled network tokens will be generated asynchronously and only used on subsequent transactions to speed up transaction processing.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.MerchantAccountCreate(
            account_updater_enabled=account_updater_enabled,
            account_updater_request_encryption_key=account_updater_request_encryption_key,
            account_updater_request_encryption_key_id=account_updater_request_encryption_key_id,
            account_updater_response_decryption_key=account_updater_response_decryption_key,
            account_updater_response_decryption_key_id=account_updater_response_decryption_key_id,
            over_capture_amount=over_capture_amount,
            over_capture_percentage=over_capture_percentage,
            loon_client_key=loon_client_key,
            loon_secret_key=loon_secret_key,
            loon_accepted_schemes=loon_accepted_schemes,
            visa_network_tokens_requestor_id=visa_network_tokens_requestor_id,
            visa_network_tokens_app_id=visa_network_tokens_app_id,
            amex_network_tokens_requestor_id=amex_network_tokens_requestor_id,
            amex_network_tokens_app_id=amex_network_tokens_app_id,
            mastercard_network_tokens_requestor_id=mastercard_network_tokens_requestor_id,
            mastercard_network_tokens_app_id=mastercard_network_tokens_app_id,
            async_network_tokens_enabled=async_network_tokens_enabled,
            id=id,
            display_name=display_name,
        )

        req = self._build_request_async(
            method="POST",
            path="/merchant-accounts",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request, False, False, "json", models.MerchantAccountCreate
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="create_merchant_account",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=[
                "400",
                "401",
                "403",
                "404",
                "405",
                "409",
                "422",
                "425",
                "429",
                "4XX",
                "500",
                "502",
                "504",
                "5XX",
            ],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "201", "application/json"):
            return unmarshal_json_response(models.MerchantAccount, http_res)
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(errors.Error400Data, http_res)
            raise errors.Error400(response_data, http_res)
        if utils.match_response(http_res, "401", "application/json"):
            response_data = unmarshal_json_response(errors.Error401Data, http_res)
            raise errors.Error401(response_data, http_res)
        if utils.match_response(http_res, "403", "application/json"):
            response_data = unmarshal_json_response(errors.Error403Data, http_res)
            raise errors.Error403(response_data, http_res)
        if utils.match_response(http_res, "404", "application/json"):
            response_data = unmarshal_json_response(errors.Error404Data, http_res)
            raise errors.Error404(response_data, http_res)
        if utils.match_response(http_res, "405", "application/json"):
            response_data = unmarshal_json_response(errors.Error405Data, http_res)
            raise errors.Error405(response_data, http_res)
        if utils.match_response(http_res, "409", "application/json"):
            response_data = unmarshal_json_response(errors.Error409Data, http_res)
            raise errors.Error409(response_data, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                errors.HTTPValidationErrorData, http_res
            )
            raise errors.HTTPValidationError(response_data, http_res)
        if utils.match_response(http_res, "425", "application/json"):
            response_data = unmarshal_json_response(errors.Error425Data, http_res)
            raise errors.Error425(response_data, http_res)
        if utils.match_response(http_res, "429", "application/json"):
            response_data = unmarshal_json_response(errors.Error429Data, http_res)
            raise errors.Error429(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(errors.Error500Data, http_res)
            raise errors.Error500(response_data, http_res)
        if utils.match_response(http_res, "502", "application/json"):
            response_data = unmarshal_json_response(errors.Error502Data, http_res)
            raise errors.Error502(response_data, http_res)
        if utils.match_response(http_res, "504", "application/json"):
            response_data = unmarshal_json_response(errors.Error504Data, http_res)
            raise errors.Error504(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)

        raise errors.APIError("Unexpected response received", http_res)

    def get(
        self,
        *,
        merchant_account_id: str,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.MerchantAccount:
        r"""Get a merchant account

        Get info about a merchant account in an instance.

        :param merchant_account_id: The ID of the merchant account
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.GetMerchantAccountRequest(
            merchant_account_id=merchant_account_id,
        )

        req = self._build_request(
            method="GET",
            path="/merchant-accounts/{merchant_account_id}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config
            else:
                retries = utils.RetryConfig(
                    "backoff", utils.BackoffStrategy(200, 200, 1, 1000), True
                )

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["5XX"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="get_merchant_account",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=[
                "400",
                "401",
                "403",
                "404",
                "405",
                "409",
                "422",
                "425",
                "429",
                "4XX",
                "500",
                "502",
                "504",
                "5XX",
            ],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(models.MerchantAccount, http_res)
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(errors.Error400Data, http_res)
            raise errors.Error400(response_data, http_res)
        if utils.match_response(http_res, "401", "application/json"):
            response_data = unmarshal_json_response(errors.Error401Data, http_res)
            raise errors.Error401(response_data, http_res)
        if utils.match_response(http_res, "403", "application/json"):
            response_data = unmarshal_json_response(errors.Error403Data, http_res)
            raise errors.Error403(response_data, http_res)
        if utils.match_response(http_res, "404", "application/json"):
            response_data = unmarshal_json_response(errors.Error404Data, http_res)
            raise errors.Error404(response_data, http_res)
        if utils.match_response(http_res, "405", "application/json"):
            response_data = unmarshal_json_response(errors.Error405Data, http_res)
            raise errors.Error405(response_data, http_res)
        if utils.match_response(http_res, "409", "application/json"):
            response_data = unmarshal_json_response(errors.Error409Data, http_res)
            raise errors.Error409(response_data, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                errors.HTTPValidationErrorData, http_res
            )
            raise errors.HTTPValidationError(response_data, http_res)
        if utils.match_response(http_res, "425", "application/json"):
            response_data = unmarshal_json_response(errors.Error425Data, http_res)
            raise errors.Error425(response_data, http_res)
        if utils.match_response(http_res, "429", "application/json"):
            response_data = unmarshal_json_response(errors.Error429Data, http_res)
            raise errors.Error429(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(errors.Error500Data, http_res)
            raise errors.Error500(response_data, http_res)
        if utils.match_response(http_res, "502", "application/json"):
            response_data = unmarshal_json_response(errors.Error502Data, http_res)
            raise errors.Error502(response_data, http_res)
        if utils.match_response(http_res, "504", "application/json"):
            response_data = unmarshal_json_response(errors.Error504Data, http_res)
            raise errors.Error504(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)

        raise errors.APIError("Unexpected response received", http_res)

    async def get_async(
        self,
        *,
        merchant_account_id: str,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.MerchantAccount:
        r"""Get a merchant account

        Get info about a merchant account in an instance.

        :param merchant_account_id: The ID of the merchant account
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.GetMerchantAccountRequest(
            merchant_account_id=merchant_account_id,
        )

        req = self._build_request_async(
            method="GET",
            path="/merchant-accounts/{merchant_account_id}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config
            else:
                retries = utils.RetryConfig(
                    "backoff", utils.BackoffStrategy(200, 200, 1, 1000), True
                )

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["5XX"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="get_merchant_account",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=[
                "400",
                "401",
                "403",
                "404",
                "405",
                "409",
                "422",
                "425",
                "429",
                "4XX",
                "500",
                "502",
                "504",
                "5XX",
            ],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(models.MerchantAccount, http_res)
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(errors.Error400Data, http_res)
            raise errors.Error400(response_data, http_res)
        if utils.match_response(http_res, "401", "application/json"):
            response_data = unmarshal_json_response(errors.Error401Data, http_res)
            raise errors.Error401(response_data, http_res)
        if utils.match_response(http_res, "403", "application/json"):
            response_data = unmarshal_json_response(errors.Error403Data, http_res)
            raise errors.Error403(response_data, http_res)
        if utils.match_response(http_res, "404", "application/json"):
            response_data = unmarshal_json_response(errors.Error404Data, http_res)
            raise errors.Error404(response_data, http_res)
        if utils.match_response(http_res, "405", "application/json"):
            response_data = unmarshal_json_response(errors.Error405Data, http_res)
            raise errors.Error405(response_data, http_res)
        if utils.match_response(http_res, "409", "application/json"):
            response_data = unmarshal_json_response(errors.Error409Data, http_res)
            raise errors.Error409(response_data, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                errors.HTTPValidationErrorData, http_res
            )
            raise errors.HTTPValidationError(response_data, http_res)
        if utils.match_response(http_res, "425", "application/json"):
            response_data = unmarshal_json_response(errors.Error425Data, http_res)
            raise errors.Error425(response_data, http_res)
        if utils.match_response(http_res, "429", "application/json"):
            response_data = unmarshal_json_response(errors.Error429Data, http_res)
            raise errors.Error429(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(errors.Error500Data, http_res)
            raise errors.Error500(response_data, http_res)
        if utils.match_response(http_res, "502", "application/json"):
            response_data = unmarshal_json_response(errors.Error502Data, http_res)
            raise errors.Error502(response_data, http_res)
        if utils.match_response(http_res, "504", "application/json"):
            response_data = unmarshal_json_response(errors.Error504Data, http_res)
            raise errors.Error504(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)

        raise errors.APIError("Unexpected response received", http_res)

    def update(
        self,
        *,
        merchant_account_id: str,
        account_updater_enabled: Optional[bool] = False,
        account_updater_request_encryption_key: OptionalNullable[str] = UNSET,
        account_updater_request_encryption_key_id: OptionalNullable[str] = UNSET,
        account_updater_response_decryption_key: OptionalNullable[str] = UNSET,
        account_updater_response_decryption_key_id: OptionalNullable[str] = UNSET,
        over_capture_amount: OptionalNullable[int] = UNSET,
        over_capture_percentage: OptionalNullable[int] = UNSET,
        loon_client_key: OptionalNullable[str] = UNSET,
        loon_secret_key: OptionalNullable[str] = UNSET,
        loon_accepted_schemes: OptionalNullable[List[models.CardScheme]] = UNSET,
        visa_network_tokens_requestor_id: OptionalNullable[str] = UNSET,
        visa_network_tokens_app_id: OptionalNullable[str] = UNSET,
        amex_network_tokens_requestor_id: OptionalNullable[str] = UNSET,
        amex_network_tokens_app_id: OptionalNullable[str] = UNSET,
        mastercard_network_tokens_requestor_id: OptionalNullable[str] = UNSET,
        mastercard_network_tokens_app_id: OptionalNullable[str] = UNSET,
        async_network_tokens_enabled: Optional[bool] = False,
        display_name: OptionalNullable[str] = UNSET,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.MerchantAccount:
        r"""Update a merchant account

        Update info for a merchant account in an instance.

        :param merchant_account_id: The ID of the merchant account
        :param account_updater_enabled: Whether the Real-Time Account Updater service is enabled for this merchant account. The Account Updater service is used to update card details when cards are lost, stolen or expired. If the field is not set or if it's set to `false`, the Account Updater service doesn't get called if a payment fails with expired or invalid card details. If the field is set to `true`, the service is called. Please note that for this to work the other `account_updater_* fields` must be set as well.
        :param account_updater_request_encryption_key: The public key used to encrypt the request to the Real-Time Account Updater service. The Account Updater service is used to update card details when cards are lost, stolen or expired. If the field is not set or if it's set to `null`, the Account Updater service doesn't get called. If the field is set, the other `account_updater_*` fields must be set as well.
        :param account_updater_request_encryption_key_id: The ID of the key used to encrypt the request to the Real-Time Account Updater service. The Account Updater service is used to update card details when cards are lost, stolen or expired. If the field is not set or if it's set to `null`, the Account Updater service doesn't get called. If the field is set, the other `account_updater_*` fields must be set as well.
        :param account_updater_response_decryption_key: The key used to decrypt the response from the Real-Time Account Updater service. The Account Updater service is used to update card details when cards are lost, stolen or expired. If the field is not set or if it's set to `null`, the Account Updater service doesn't get called. If the field is set, the other `account_updater_*` fields must be set as well.
        :param account_updater_response_decryption_key_id: The ID of the key used to decrypt the request from the Real-Time Account Updater service. The Account Updater service is used to update card details when cards are lost, stolen or expired. If the field is not set or if it's set to `null`, the Account Updater service doesn't get called. If the field is set, the other `account_updater_*` fields must be set as well.
        :param over_capture_amount: The maximum monetary amount allowed for over-capture, in the smallest currency unit, for example `1299` cents to allow for an over-capture of `$12.99`.
        :param over_capture_percentage: The maximum percentage allowed for over-capture, for example `25` to allow for an over-capture of `25%` of the original transaction amount.
        :param loon_client_key: Client key provided by Pagos to authenticate to the Loon API. Loon is the Account Updater service we use and if the field is not set or if it's set to null, the Account Updater service doesn't get configured. If the field is set to `null`, the other `loon_*` fields must be set to null as well.
        :param loon_secret_key: Secret key provided by Pagos to authenticate to the Loon API. Loon is the Account Updater service we use and if the field is not set or if it's set to null, the Account Updater service doesn't get configured. If the field is set to `null`, the other `loon_*` fields must be set to null as well.
        :param loon_accepted_schemes: Card schemes accepted when creating jobs using this set of Loon API keys. Loon is the Account Updater service we use and if the field is not set or if it's set to null, the Account Updater service doesn't get configured. If the field is set to `null`, the other `loon_*` fields must be set to null as well.
        :param visa_network_tokens_requestor_id: Requestor ID provided for Visa after onboarding to use Network Tokens.
        :param visa_network_tokens_app_id: Application ID provided for Visa after onboarding to use Network Tokens.
        :param amex_network_tokens_requestor_id: Requestor ID provided for American Express after onboarding to use Network Tokens.
        :param amex_network_tokens_app_id: Application ID provided for American Express after onboarding to use Network Tokens.
        :param mastercard_network_tokens_requestor_id: Requestor ID provided for Mastercard after onboarding to use Network Tokens.
        :param mastercard_network_tokens_app_id: Application ID provided for Mastercard after onboarding to use Network Tokens.
        :param async_network_tokens_enabled: When enabled network tokens will be generated asynchronously and only used on subsequent transactions to speed up transaction processing.
        :param display_name: The display name for the merchant account.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.UpdateMerchantAccountRequest(
            merchant_account_id=merchant_account_id,
            merchant_account_update=models.MerchantAccountUpdate(
                account_updater_enabled=account_updater_enabled,
                account_updater_request_encryption_key=account_updater_request_encryption_key,
                account_updater_request_encryption_key_id=account_updater_request_encryption_key_id,
                account_updater_response_decryption_key=account_updater_response_decryption_key,
                account_updater_response_decryption_key_id=account_updater_response_decryption_key_id,
                over_capture_amount=over_capture_amount,
                over_capture_percentage=over_capture_percentage,
                loon_client_key=loon_client_key,
                loon_secret_key=loon_secret_key,
                loon_accepted_schemes=loon_accepted_schemes,
                visa_network_tokens_requestor_id=visa_network_tokens_requestor_id,
                visa_network_tokens_app_id=visa_network_tokens_app_id,
                amex_network_tokens_requestor_id=amex_network_tokens_requestor_id,
                amex_network_tokens_app_id=amex_network_tokens_app_id,
                mastercard_network_tokens_requestor_id=mastercard_network_tokens_requestor_id,
                mastercard_network_tokens_app_id=mastercard_network_tokens_app_id,
                async_network_tokens_enabled=async_network_tokens_enabled,
                display_name=display_name,
            ),
        )

        req = self._build_request(
            method="PUT",
            path="/merchant-accounts/{merchant_account_id}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.merchant_account_update,
                False,
                False,
                "json",
                models.MerchantAccountUpdate,
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="update_merchant_account",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=[
                "400",
                "401",
                "403",
                "404",
                "405",
                "409",
                "422",
                "425",
                "429",
                "4XX",
                "500",
                "502",
                "504",
                "5XX",
            ],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(models.MerchantAccount, http_res)
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(errors.Error400Data, http_res)
            raise errors.Error400(response_data, http_res)
        if utils.match_response(http_res, "401", "application/json"):
            response_data = unmarshal_json_response(errors.Error401Data, http_res)
            raise errors.Error401(response_data, http_res)
        if utils.match_response(http_res, "403", "application/json"):
            response_data = unmarshal_json_response(errors.Error403Data, http_res)
            raise errors.Error403(response_data, http_res)
        if utils.match_response(http_res, "404", "application/json"):
            response_data = unmarshal_json_response(errors.Error404Data, http_res)
            raise errors.Error404(response_data, http_res)
        if utils.match_response(http_res, "405", "application/json"):
            response_data = unmarshal_json_response(errors.Error405Data, http_res)
            raise errors.Error405(response_data, http_res)
        if utils.match_response(http_res, "409", "application/json"):
            response_data = unmarshal_json_response(errors.Error409Data, http_res)
            raise errors.Error409(response_data, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                errors.HTTPValidationErrorData, http_res
            )
            raise errors.HTTPValidationError(response_data, http_res)
        if utils.match_response(http_res, "425", "application/json"):
            response_data = unmarshal_json_response(errors.Error425Data, http_res)
            raise errors.Error425(response_data, http_res)
        if utils.match_response(http_res, "429", "application/json"):
            response_data = unmarshal_json_response(errors.Error429Data, http_res)
            raise errors.Error429(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(errors.Error500Data, http_res)
            raise errors.Error500(response_data, http_res)
        if utils.match_response(http_res, "502", "application/json"):
            response_data = unmarshal_json_response(errors.Error502Data, http_res)
            raise errors.Error502(response_data, http_res)
        if utils.match_response(http_res, "504", "application/json"):
            response_data = unmarshal_json_response(errors.Error504Data, http_res)
            raise errors.Error504(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)

        raise errors.APIError("Unexpected response received", http_res)

    async def update_async(
        self,
        *,
        merchant_account_id: str,
        account_updater_enabled: Optional[bool] = False,
        account_updater_request_encryption_key: OptionalNullable[str] = UNSET,
        account_updater_request_encryption_key_id: OptionalNullable[str] = UNSET,
        account_updater_response_decryption_key: OptionalNullable[str] = UNSET,
        account_updater_response_decryption_key_id: OptionalNullable[str] = UNSET,
        over_capture_amount: OptionalNullable[int] = UNSET,
        over_capture_percentage: OptionalNullable[int] = UNSET,
        loon_client_key: OptionalNullable[str] = UNSET,
        loon_secret_key: OptionalNullable[str] = UNSET,
        loon_accepted_schemes: OptionalNullable[List[models.CardScheme]] = UNSET,
        visa_network_tokens_requestor_id: OptionalNullable[str] = UNSET,
        visa_network_tokens_app_id: OptionalNullable[str] = UNSET,
        amex_network_tokens_requestor_id: OptionalNullable[str] = UNSET,
        amex_network_tokens_app_id: OptionalNullable[str] = UNSET,
        mastercard_network_tokens_requestor_id: OptionalNullable[str] = UNSET,
        mastercard_network_tokens_app_id: OptionalNullable[str] = UNSET,
        async_network_tokens_enabled: Optional[bool] = False,
        display_name: OptionalNullable[str] = UNSET,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.MerchantAccount:
        r"""Update a merchant account

        Update info for a merchant account in an instance.

        :param merchant_account_id: The ID of the merchant account
        :param account_updater_enabled: Whether the Real-Time Account Updater service is enabled for this merchant account. The Account Updater service is used to update card details when cards are lost, stolen or expired. If the field is not set or if it's set to `false`, the Account Updater service doesn't get called if a payment fails with expired or invalid card details. If the field is set to `true`, the service is called. Please note that for this to work the other `account_updater_* fields` must be set as well.
        :param account_updater_request_encryption_key: The public key used to encrypt the request to the Real-Time Account Updater service. The Account Updater service is used to update card details when cards are lost, stolen or expired. If the field is not set or if it's set to `null`, the Account Updater service doesn't get called. If the field is set, the other `account_updater_*` fields must be set as well.
        :param account_updater_request_encryption_key_id: The ID of the key used to encrypt the request to the Real-Time Account Updater service. The Account Updater service is used to update card details when cards are lost, stolen or expired. If the field is not set or if it's set to `null`, the Account Updater service doesn't get called. If the field is set, the other `account_updater_*` fields must be set as well.
        :param account_updater_response_decryption_key: The key used to decrypt the response from the Real-Time Account Updater service. The Account Updater service is used to update card details when cards are lost, stolen or expired. If the field is not set or if it's set to `null`, the Account Updater service doesn't get called. If the field is set, the other `account_updater_*` fields must be set as well.
        :param account_updater_response_decryption_key_id: The ID of the key used to decrypt the request from the Real-Time Account Updater service. The Account Updater service is used to update card details when cards are lost, stolen or expired. If the field is not set or if it's set to `null`, the Account Updater service doesn't get called. If the field is set, the other `account_updater_*` fields must be set as well.
        :param over_capture_amount: The maximum monetary amount allowed for over-capture, in the smallest currency unit, for example `1299` cents to allow for an over-capture of `$12.99`.
        :param over_capture_percentage: The maximum percentage allowed for over-capture, for example `25` to allow for an over-capture of `25%` of the original transaction amount.
        :param loon_client_key: Client key provided by Pagos to authenticate to the Loon API. Loon is the Account Updater service we use and if the field is not set or if it's set to null, the Account Updater service doesn't get configured. If the field is set to `null`, the other `loon_*` fields must be set to null as well.
        :param loon_secret_key: Secret key provided by Pagos to authenticate to the Loon API. Loon is the Account Updater service we use and if the field is not set or if it's set to null, the Account Updater service doesn't get configured. If the field is set to `null`, the other `loon_*` fields must be set to null as well.
        :param loon_accepted_schemes: Card schemes accepted when creating jobs using this set of Loon API keys. Loon is the Account Updater service we use and if the field is not set or if it's set to null, the Account Updater service doesn't get configured. If the field is set to `null`, the other `loon_*` fields must be set to null as well.
        :param visa_network_tokens_requestor_id: Requestor ID provided for Visa after onboarding to use Network Tokens.
        :param visa_network_tokens_app_id: Application ID provided for Visa after onboarding to use Network Tokens.
        :param amex_network_tokens_requestor_id: Requestor ID provided for American Express after onboarding to use Network Tokens.
        :param amex_network_tokens_app_id: Application ID provided for American Express after onboarding to use Network Tokens.
        :param mastercard_network_tokens_requestor_id: Requestor ID provided for Mastercard after onboarding to use Network Tokens.
        :param mastercard_network_tokens_app_id: Application ID provided for Mastercard after onboarding to use Network Tokens.
        :param async_network_tokens_enabled: When enabled network tokens will be generated asynchronously and only used on subsequent transactions to speed up transaction processing.
        :param display_name: The display name for the merchant account.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.UpdateMerchantAccountRequest(
            merchant_account_id=merchant_account_id,
            merchant_account_update=models.MerchantAccountUpdate(
                account_updater_enabled=account_updater_enabled,
                account_updater_request_encryption_key=account_updater_request_encryption_key,
                account_updater_request_encryption_key_id=account_updater_request_encryption_key_id,
                account_updater_response_decryption_key=account_updater_response_decryption_key,
                account_updater_response_decryption_key_id=account_updater_response_decryption_key_id,
                over_capture_amount=over_capture_amount,
                over_capture_percentage=over_capture_percentage,
                loon_client_key=loon_client_key,
                loon_secret_key=loon_secret_key,
                loon_accepted_schemes=loon_accepted_schemes,
                visa_network_tokens_requestor_id=visa_network_tokens_requestor_id,
                visa_network_tokens_app_id=visa_network_tokens_app_id,
                amex_network_tokens_requestor_id=amex_network_tokens_requestor_id,
                amex_network_tokens_app_id=amex_network_tokens_app_id,
                mastercard_network_tokens_requestor_id=mastercard_network_tokens_requestor_id,
                mastercard_network_tokens_app_id=mastercard_network_tokens_app_id,
                async_network_tokens_enabled=async_network_tokens_enabled,
                display_name=display_name,
            ),
        )

        req = self._build_request_async(
            method="PUT",
            path="/merchant-accounts/{merchant_account_id}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.merchant_account_update,
                False,
                False,
                "json",
                models.MerchantAccountUpdate,
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="update_merchant_account",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=[
                "400",
                "401",
                "403",
                "404",
                "405",
                "409",
                "422",
                "425",
                "429",
                "4XX",
                "500",
                "502",
                "504",
                "5XX",
            ],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(models.MerchantAccount, http_res)
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(errors.Error400Data, http_res)
            raise errors.Error400(response_data, http_res)
        if utils.match_response(http_res, "401", "application/json"):
            response_data = unmarshal_json_response(errors.Error401Data, http_res)
            raise errors.Error401(response_data, http_res)
        if utils.match_response(http_res, "403", "application/json"):
            response_data = unmarshal_json_response(errors.Error403Data, http_res)
            raise errors.Error403(response_data, http_res)
        if utils.match_response(http_res, "404", "application/json"):
            response_data = unmarshal_json_response(errors.Error404Data, http_res)
            raise errors.Error404(response_data, http_res)
        if utils.match_response(http_res, "405", "application/json"):
            response_data = unmarshal_json_response(errors.Error405Data, http_res)
            raise errors.Error405(response_data, http_res)
        if utils.match_response(http_res, "409", "application/json"):
            response_data = unmarshal_json_response(errors.Error409Data, http_res)
            raise errors.Error409(response_data, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                errors.HTTPValidationErrorData, http_res
            )
            raise errors.HTTPValidationError(response_data, http_res)
        if utils.match_response(http_res, "425", "application/json"):
            response_data = unmarshal_json_response(errors.Error425Data, http_res)
            raise errors.Error425(response_data, http_res)
        if utils.match_response(http_res, "429", "application/json"):
            response_data = unmarshal_json_response(errors.Error429Data, http_res)
            raise errors.Error429(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(errors.Error500Data, http_res)
            raise errors.Error500(response_data, http_res)
        if utils.match_response(http_res, "502", "application/json"):
            response_data = unmarshal_json_response(errors.Error502Data, http_res)
            raise errors.Error502(response_data, http_res)
        if utils.match_response(http_res, "504", "application/json"):
            response_data = unmarshal_json_response(errors.Error504Data, http_res)
            raise errors.Error504(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)

        raise errors.APIError("Unexpected response received", http_res)



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .accountupdateroptions import AccountUpdaterOptions, AccountUpdaterOptionsTypedDict
from .adyencardoptions import AdyenCardOptions, AdyenCardOptionsTypedDict
from .adyenoptions import AdyenOptions, AdyenOptionsTypedDict
from .adyensepaoptions import AdyenSepaOptions, AdyenSepaOptionsTypedDict
from .affirmoptions import AffirmOptions, AffirmOptionsTypedDict
from .braintreeoptions import BraintreeOptions, BraintreeOptionsTypedDict
from .cybersourceantifraudoptions import (
    CybersourceAntiFraudOptions,
    CybersourceAntiFraudOptionsTypedDict,
)
from .cybersourceoptions import CybersourceOptions, CybersourceOptionsTypedDict
from .dlocaloptions import DlocalOptions, DlocalOptionsTypedDict
from .dlocalupioptions import DlocalUPIOptions, DlocalUPIOptionsTypedDict
from .fiservoptions import FiservOptions, FiservOptionsTypedDict
from .forterantifraudoptions import (
    ForterAntiFraudOptions,
    ForterAntiFraudOptionsTypedDict,
)
from .givingblockoptions import GivingBlockOptions, GivingBlockOptionsTypedDict
from .latitudeoptions import LatitudeOptions, LatitudeOptionsTypedDict
from .mattildatapioptions import MattildaTapiOptions, MattildaTapiOptionsTypedDict
from .mockcardoptions import MockCardOptions, MockCardOptionsTypedDict
from .nuveioptions import NuveiOptions, NuveiOptionsTypedDict
from .nuveipseoptions import NuveiPSEOptions, NuveiPSEOptionsTypedDict
from .oxxooptions import OxxoOptions, OxxoOptionsTypedDict
from .paypaloptions import PaypalOptions, PaypalOptionsTypedDict
from .powertranzoptions import PowertranzOptions, PowertranzOptionsTypedDict
from .stripeoptions import StripeOptions, StripeOptionsTypedDict
from .travelhuboptions import TravelhubOptions, TravelhubOptionsTypedDict
from .trustlyoptions import TrustlyOptions, TrustlyOptionsTypedDict
from .wpayeverdaypayoptions import WpayEverdaypayOptions, WpayEverdaypayOptionsTypedDict
from .wpaypaytooptions import WpayPaytoOptions, WpayPaytoOptionsTypedDict
from gr4vy.types import BaseModel, Nullable, OptionalNullable, UNSET, UNSET_SENTINEL
import pydantic
from pydantic import model_serializer
from typing_extensions import Annotated, NotRequired, TypedDict


class TransactionConnectionOptionsTypedDict(TypedDict):
    account_updater: NotRequired[Nullable[AccountUpdaterOptionsTypedDict]]
    r"""Custom options to be passed to the `account-updater` connector, allowing for simulating different account updater responses."""
    adyen_afterpay: NotRequired[Nullable[AdyenOptionsTypedDict]]
    r"""Custom options to be passed to the `adyen-afterpay` connector."""
    adyen_alipay: NotRequired[Nullable[AdyenOptionsTypedDict]]
    r"""Custom options to be passed to the `adyen-alipay` connector."""
    adyen_card: NotRequired[Nullable[AdyenCardOptionsTypedDict]]
    r"""Custom options to be passed to the `adyen-card` connector."""
    adyen_cashapp: NotRequired[Nullable[AdyenOptionsTypedDict]]
    r"""Custom options to be passed to the `adyen-cashapp` connector."""
    adyen_giropay: NotRequired[Nullable[AdyenOptionsTypedDict]]
    r"""Custom options to be passed to the `adyen-giropay` connector."""
    adyen_ideal: NotRequired[Nullable[AdyenOptionsTypedDict]]
    r"""Custom options to be passed to the `adyen-ideal` connector."""
    adyen_sepa: NotRequired[Nullable[AdyenSepaOptionsTypedDict]]
    r"""Custom options to be passed to the `adyen-sepa` connector."""
    adyen_sofort: NotRequired[Nullable[AdyenOptionsTypedDict]]
    r"""Custom options to be passed to the `adyen-sofort` connector."""
    adyen_vipps: NotRequired[Nullable[AdyenOptionsTypedDict]]
    r"""Custom options to be passed to the `adyen-vipps` connector."""
    affirm_affirm: NotRequired[Nullable[AffirmOptionsTypedDict]]
    r"""Custom options to be passed to the `affirm-affirm` connector."""
    braintree_card: NotRequired[Nullable[BraintreeOptionsTypedDict]]
    r"""Custom options to be passed to the `braintree-card` connector."""
    cybersource_anti_fraud: NotRequired[Nullable[CybersourceAntiFraudOptionsTypedDict]]
    r"""Custom options to be passed to the `cybersource-anti-fraud` connector."""
    cybersource_card: NotRequired[Nullable[CybersourceOptionsTypedDict]]
    r"""Custom options to be passed to the `cybersource-card` connector."""
    cybersource_ideal: NotRequired[Nullable[CybersourceOptionsTypedDict]]
    r"""Custom options to be passed to the `cybersource-ideal` connector."""
    cybersource_kcp: NotRequired[Nullable[CybersourceOptionsTypedDict]]
    r"""Custom options to be passed to the `cybersource-kcp` connector."""
    dlocal_nequi: NotRequired[Nullable[DlocalOptionsTypedDict]]
    r"""Custom options to be passed to the `dlocal-nequi` connector."""
    dlocal_upi: NotRequired[Nullable[DlocalUPIOptionsTypedDict]]
    r"""Custom options to be passed to the `dlocal-upi` connector."""
    fiserv_card: NotRequired[Nullable[FiservOptionsTypedDict]]
    r"""Custom options to be passed to the `fiserv-card` connector."""
    forter_anti_fraud: NotRequired[Nullable[ForterAntiFraudOptionsTypedDict]]
    r"""Custom options to be passed to the `forter-anti-fraud` connector."""
    gem_gem: NotRequired[Nullable[LatitudeOptionsTypedDict]]
    r"""Custom options to be passed to the `gem-gem` connector."""
    gem_gemds: NotRequired[Nullable[LatitudeOptionsTypedDict]]
    r"""Custom options to be passed to the `gem-gemds` connector."""
    givingblock_givingblock: NotRequired[Nullable[GivingBlockOptionsTypedDict]]
    r"""Custom options to be passed to the `givingblock-givingblock` connector."""
    latitude_latitude: NotRequired[Nullable[LatitudeOptionsTypedDict]]
    r"""Custom options to be passed to the `latitude-latitude` connector."""
    latitude_latitudeds: NotRequired[Nullable[LatitudeOptionsTypedDict]]
    r"""Custom options to be passed to the `latitude-latitudeds` connector."""
    mattilda_tapi: NotRequired[Nullable[MattildaTapiOptionsTypedDict]]
    r"""Custom options to be passed to the `mattilda-tapi` connector."""
    mattilda_tapifintechs: NotRequired[Nullable[MattildaTapiOptionsTypedDict]]
    r"""Custom options to be passed to the `mattilda-tapifintechs` connector."""
    mock_card: NotRequired[Nullable[MockCardOptionsTypedDict]]
    r"""Custom options to be passed to the `mock-card` connector."""
    nuvei_card: NotRequired[Nullable[NuveiOptionsTypedDict]]
    r"""Custom options to be passed to the `nuvei-card` connector."""
    nuvei_pse: NotRequired[Nullable[NuveiPSEOptionsTypedDict]]
    r"""Custom options to be passed to the `nuvei-pse` connector."""
    oxxo_oxxo: NotRequired[Nullable[OxxoOptionsTypedDict]]
    r"""Custom options to be passed to the `oxxo-oxxo` connector."""
    paypal_paypal: NotRequired[Nullable[PaypalOptionsTypedDict]]
    r"""Custom options to be passed to the `paypal-paypal` connector."""
    paypal_paypalpaylater: NotRequired[Nullable[PaypalOptionsTypedDict]]
    r"""Custom options to be passed to the `paypal-paypalpaylater` connector."""
    powertranz_card: NotRequired[Nullable[PowertranzOptionsTypedDict]]
    r"""Custom options to be passed to the `powertranz-card` connector."""
    stripe_card: NotRequired[Nullable[StripeOptionsTypedDict]]
    r"""Custom options to be passed to the `stripe-card` connector."""
    travelhub_card: NotRequired[Nullable[TravelhubOptionsTypedDict]]
    r"""Custom options to be passed to the `travelhub-card` connector."""
    trustly_trustly: NotRequired[Nullable[TrustlyOptionsTypedDict]]
    r"""Custom options to be passed to the `trustly-trustly` connector."""
    wpay_everydaypay: NotRequired[Nullable[WpayEverdaypayOptionsTypedDict]]
    r"""Custom options to be passed to the `wpay-everydaypay` connector."""
    wpay_payto: NotRequired[Nullable[WpayPaytoOptionsTypedDict]]
    r"""Custom options to be passed to the `wpay-payto` connector."""


class TransactionConnectionOptions(BaseModel):
    account_updater: Annotated[
        OptionalNullable[AccountUpdaterOptions], pydantic.Field(alias="account-updater")
    ] = UNSET
    r"""Custom options to be passed to the `account-updater` connector, allowing for simulating different account updater responses."""

    adyen_afterpay: Annotated[
        OptionalNullable[AdyenOptions], pydantic.Field(alias="adyen-afterpay")
    ] = UNSET
    r"""Custom options to be passed to the `adyen-afterpay` connector."""

    adyen_alipay: Annotated[
        OptionalNullable[AdyenOptions], pydantic.Field(alias="adyen-alipay")
    ] = UNSET
    r"""Custom options to be passed to the `adyen-alipay` connector."""

    adyen_card: Annotated[
        OptionalNullable[AdyenCardOptions], pydantic.Field(alias="adyen-card")
    ] = UNSET
    r"""Custom options to be passed to the `adyen-card` connector."""

    adyen_cashapp: Annotated[
        OptionalNullable[AdyenOptions], pydantic.Field(alias="adyen-cashapp")
    ] = UNSET
    r"""Custom options to be passed to the `adyen-cashapp` connector."""

    adyen_giropay: Annotated[
        OptionalNullable[AdyenOptions], pydantic.Field(alias="adyen-giropay")
    ] = UNSET
    r"""Custom options to be passed to the `adyen-giropay` connector."""

    adyen_ideal: Annotated[
        OptionalNullable[AdyenOptions], pydantic.Field(alias="adyen-ideal")
    ] = UNSET
    r"""Custom options to be passed to the `adyen-ideal` connector."""

    adyen_sepa: Annotated[
        OptionalNullable[AdyenSepaOptions], pydantic.Field(alias="adyen-sepa")
    ] = UNSET
    r"""Custom options to be passed to the `adyen-sepa` connector."""

    adyen_sofort: Annotated[
        OptionalNullable[AdyenOptions], pydantic.Field(alias="adyen-sofort")
    ] = UNSET
    r"""Custom options to be passed to the `adyen-sofort` connector."""

    adyen_vipps: Annotated[
        OptionalNullable[AdyenOptions], pydantic.Field(alias="adyen-vipps")
    ] = UNSET
    r"""Custom options to be passed to the `adyen-vipps` connector."""

    affirm_affirm: Annotated[
        OptionalNullable[AffirmOptions], pydantic.Field(alias="affirm-affirm")
    ] = UNSET
    r"""Custom options to be passed to the `affirm-affirm` connector."""

    braintree_card: Annotated[
        OptionalNullable[BraintreeOptions], pydantic.Field(alias="braintree-card")
    ] = UNSET
    r"""Custom options to be passed to the `braintree-card` connector."""

    cybersource_anti_fraud: Annotated[
        OptionalNullable[CybersourceAntiFraudOptions],
        pydantic.Field(alias="cybersource-anti-fraud"),
    ] = UNSET
    r"""Custom options to be passed to the `cybersource-anti-fraud` connector."""

    cybersource_card: Annotated[
        OptionalNullable[CybersourceOptions], pydantic.Field(alias="cybersource-card")
    ] = UNSET
    r"""Custom options to be passed to the `cybersource-card` connector."""

    cybersource_ideal: Annotated[
        OptionalNullable[CybersourceOptions], pydantic.Field(alias="cybersource-ideal")
    ] = UNSET
    r"""Custom options to be passed to the `cybersource-ideal` connector."""

    cybersource_kcp: Annotated[
        OptionalNullable[CybersourceOptions], pydantic.Field(alias="cybersource-kcp")
    ] = UNSET
    r"""Custom options to be passed to the `cybersource-kcp` connector."""

    dlocal_nequi: Annotated[
        OptionalNullable[DlocalOptions], pydantic.Field(alias="dlocal-nequi")
    ] = UNSET
    r"""Custom options to be passed to the `dlocal-nequi` connector."""

    dlocal_upi: Annotated[
        OptionalNullable[DlocalUPIOptions], pydantic.Field(alias="dlocal-upi")
    ] = UNSET
    r"""Custom options to be passed to the `dlocal-upi` connector."""

    fiserv_card: Annotated[
        OptionalNullable[FiservOptions], pydantic.Field(alias="fiserv-card")
    ] = UNSET
    r"""Custom options to be passed to the `fiserv-card` connector."""

    forter_anti_fraud: Annotated[
        OptionalNullable[ForterAntiFraudOptions],
        pydantic.Field(alias="forter-anti-fraud"),
    ] = UNSET
    r"""Custom options to be passed to the `forter-anti-fraud` connector."""

    gem_gem: Annotated[
        OptionalNullable[LatitudeOptions], pydantic.Field(alias="gem-gem")
    ] = UNSET
    r"""Custom options to be passed to the `gem-gem` connector."""

    gem_gemds: Annotated[
        OptionalNullable[LatitudeOptions], pydantic.Field(alias="gem-gemds")
    ] = UNSET
    r"""Custom options to be passed to the `gem-gemds` connector."""

    givingblock_givingblock: Annotated[
        OptionalNullable[GivingBlockOptions],
        pydantic.Field(alias="givingblock-givingblock"),
    ] = UNSET
    r"""Custom options to be passed to the `givingblock-givingblock` connector."""

    latitude_latitude: Annotated[
        OptionalNullable[LatitudeOptions], pydantic.Field(alias="latitude-latitude")
    ] = UNSET
    r"""Custom options to be passed to the `latitude-latitude` connector."""

    latitude_latitudeds: Annotated[
        OptionalNullable[LatitudeOptions], pydantic.Field(alias="latitude-latitudeds")
    ] = UNSET
    r"""Custom options to be passed to the `latitude-latitudeds` connector."""

    mattilda_tapi: Annotated[
        OptionalNullable[MattildaTapiOptions], pydantic.Field(alias="mattilda-tapi")
    ] = UNSET
    r"""Custom options to be passed to the `mattilda-tapi` connector."""

    mattilda_tapifintechs: Annotated[
        OptionalNullable[MattildaTapiOptions],
        pydantic.Field(alias="mattilda-tapifintechs"),
    ] = UNSET
    r"""Custom options to be passed to the `mattilda-tapifintechs` connector."""

    mock_card: Annotated[
        OptionalNullable[MockCardOptions], pydantic.Field(alias="mock-card")
    ] = UNSET
    r"""Custom options to be passed to the `mock-card` connector."""

    nuvei_card: Annotated[
        OptionalNullable[NuveiOptions], pydantic.Field(alias="nuvei-card")
    ] = UNSET
    r"""Custom options to be passed to the `nuvei-card` connector."""

    nuvei_pse: Annotated[
        OptionalNullable[NuveiPSEOptions], pydantic.Field(alias="nuvei-pse")
    ] = UNSET
    r"""Custom options to be passed to the `nuvei-pse` connector."""

    oxxo_oxxo: Annotated[
        OptionalNullable[OxxoOptions], pydantic.Field(alias="oxxo-oxxo")
    ] = UNSET
    r"""Custom options to be passed to the `oxxo-oxxo` connector."""

    paypal_paypal: Annotated[
        OptionalNullable[PaypalOptions], pydantic.Field(alias="paypal-paypal")
    ] = UNSET
    r"""Custom options to be passed to the `paypal-paypal` connector."""

    paypal_paypalpaylater: Annotated[
        OptionalNullable[PaypalOptions], pydantic.Field(alias="paypal-paypalpaylater")
    ] = UNSET
    r"""Custom options to be passed to the `paypal-paypalpaylater` connector."""

    powertranz_card: Annotated[
        OptionalNullable[PowertranzOptions], pydantic.Field(alias="powertranz-card")
    ] = UNSET
    r"""Custom options to be passed to the `powertranz-card` connector."""

    stripe_card: Annotated[
        OptionalNullable[StripeOptions], pydantic.Field(alias="stripe-card")
    ] = UNSET
    r"""Custom options to be passed to the `stripe-card` connector."""

    travelhub_card: Annotated[
        OptionalNullable[TravelhubOptions], pydantic.Field(alias="travelhub-card")
    ] = UNSET
    r"""Custom options to be passed to the `travelhub-card` connector."""

    trustly_trustly: Annotated[
        OptionalNullable[TrustlyOptions], pydantic.Field(alias="trustly-trustly")
    ] = UNSET
    r"""Custom options to be passed to the `trustly-trustly` connector."""

    wpay_everydaypay: Annotated[
        OptionalNullable[WpayEverdaypayOptions],
        pydantic.Field(alias="wpay-everydaypay"),
    ] = UNSET
    r"""Custom options to be passed to the `wpay-everydaypay` connector."""

    wpay_payto: Annotated[
        OptionalNullable[WpayPaytoOptions], pydantic.Field(alias="wpay-payto")
    ] = UNSET
    r"""Custom options to be passed to the `wpay-payto` connector."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "account-updater",
            "adyen-afterpay",
            "adyen-alipay",
            "adyen-card",
            "adyen-cashapp",
            "adyen-giropay",
            "adyen-ideal",
            "adyen-sepa",
            "adyen-sofort",
            "adyen-vipps",
            "affirm-affirm",
            "braintree-card",
            "cybersource-anti-fraud",
            "cybersource-card",
            "cybersource-ideal",
            "cybersource-kcp",
            "dlocal-nequi",
            "dlocal-upi",
            "fiserv-card",
            "forter-anti-fraud",
            "gem-gem",
            "gem-gemds",
            "givingblock-givingblock",
            "latitude-latitude",
            "latitude-latitudeds",
            "mattilda-tapi",
            "mattilda-tapifintechs",
            "mock-card",
            "nuvei-card",
            "nuvei-pse",
            "oxxo-oxxo",
            "paypal-paypal",
            "paypal-paypalpaylater",
            "powertranz-card",
            "stripe-card",
            "travelhub-card",
            "trustly-trustly",
            "wpay-everydaypay",
            "wpay-payto",
        ]
        nullable_fields = [
            "account-updater",
            "adyen-afterpay",
            "adyen-alipay",
            "adyen-card",
            "adyen-cashapp",
            "adyen-giropay",
            "adyen-ideal",
            "adyen-sepa",
            "adyen-sofort",
            "adyen-vipps",
            "affirm-affirm",
            "braintree-card",
            "cybersource-anti-fraud",
            "cybersource-card",
            "cybersource-ideal",
            "cybersource-kcp",
            "dlocal-nequi",
            "dlocal-upi",
            "fiserv-card",
            "forter-anti-fraud",
            "gem-gem",
            "gem-gemds",
            "givingblock-givingblock",
            "latitude-latitude",
            "latitude-latitudeds",
            "mattilda-tapi",
            "mattilda-tapifintechs",
            "mock-card",
            "nuvei-card",
            "nuvei-pse",
            "oxxo-oxxo",
            "paypal-paypal",
            "paypal-paypalpaylater",
            "powertranz-card",
            "stripe-card",
            "travelhub-card",
            "trustly-trustly",
            "wpay-everydaypay",
            "wpay-payto",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from gr4vy.types import UnrecognizedStr
from typing import Literal, Union


RefundTargetType = Union[
    Literal[
        "payment-method",
        "gift-card-redemption",
    ],
    UnrecognizedStr,
]



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .giftcardrequest import GiftCardRequest, GiftCardRequestTypedDict
from .giftcardstoredrequest import GiftCardStoredRequest, GiftCardStoredRequestTypedDict
from gr4vy.types import BaseModel
from typing import List, Union
from typing_extensions import TypeAliasType, TypedDict


ItemTypedDict = TypeAliasType(
    "ItemTypedDict", Union[GiftCardStoredRequestTypedDict, GiftCardRequestTypedDict]
)


Item = TypeAliasType("Item", Union[GiftCardStoredRequest, GiftCardRequest])


class GiftCardBalanceRequestTypedDict(TypedDict):
    items: List[ItemTypedDict]
    r"""A list of gift cards to request a balance for."""


class GiftCardBalanceRequest(BaseModel):
    items: List[Item]
    r"""A list of gift cards to request a balance for."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from gr4vy.types import UnrecognizedStr
from typing import Literal, Union


Mode = Union[
    Literal[
        "card",
        "redirect",
        "applepay",
        "googlepay",
        "checkout-session",
        "click-to-pay",
        "gift-card",
    ],
    UnrecognizedStr,
]



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .airline import Airline, AirlineTypedDict
from .antifrauddecision import AntiFraudDecision
from .avsresponsecode import AVSResponseCode
from .cartitem import CartItem, CartItemTypedDict
from .cvvresponsecode import CVVResponseCode
from .giftcardredemption import GiftCardRedemption, GiftCardRedemptionTypedDict
from .giftcardservice import GiftCardService, GiftCardServiceTypedDict
from .instrumenttype import InstrumentType
from .method import Method
from .recipient import Recipient, RecipientTypedDict
from .shippingdetails import ShippingDetails, ShippingDetailsTypedDict
from .statementdescriptor import StatementDescriptor, StatementDescriptorTypedDict
from .transactionbuyer import TransactionBuyer, TransactionBuyerTypedDict
from .transactionintent import TransactionIntent
from .transactionintentoutcome import TransactionIntentOutcome
from .transactionpaymentmethod import (
    TransactionPaymentMethod,
    TransactionPaymentMethodTypedDict,
)
from .transactionpaymentservice import (
    TransactionPaymentService,
    TransactionPaymentServiceTypedDict,
)
from .transactionpaymentsource import TransactionPaymentSource
from .transactionstatus import TransactionStatus
from .transactionthreedsecuresummary import (
    TransactionThreeDSecureSummary,
    TransactionThreeDSecureSummaryTypedDict,
)
from datetime import datetime
from gr4vy.types import BaseModel, Nullable, OptionalNullable, UNSET, UNSET_SENTINEL
from gr4vy.utils import validate_const, validate_open_enum
import pydantic
from pydantic import model_serializer
from pydantic.functional_validators import AfterValidator, PlainValidator
from typing import Dict, List, Literal, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class TransactionTypedDict(TypedDict):
    r"""A full transaction resource."""

    id: str
    r"""The ID for the transaction."""
    reconciliation_id: str
    r"""The base62 encoded transaction ID. This represents a shorter version of this transaction's `id` which is sent to payment services, anti-fraud services, and other connectors. You can use this ID to reconcile a payment service's transaction against our system. This ID is sent instead of the transaction ID because not all services support 36 digit identifiers."""
    merchant_account_id: str
    r"""The ID of the merchant account this transaction belongs to."""
    currency: str
    r"""The currency code for this transaction."""
    amount: int
    r"""The total amount for this transaction across all funding sources including gift cards."""
    status: TransactionStatus
    authorized_amount: int
    r"""The amount for this transaction that has been authorized for the `payment_method`. This can be less than the `amount` if gift cards were used."""
    captured_amount: int
    r"""The total amount captured for this transaction, in the smallest currency unit (for example, cents or pence). This can be the full value of the `authorized_amount` or less."""
    refunded_amount: int
    r"""The total amount refunded for this transaction, in the smallest currency unit (for example, cents or pence). This can be the full value of the `captured_amount` or less."""
    settled_amount: int
    r"""The net amount settled for this transaction, in the smallest currency unit (for example, cents or pence)."""
    settled: bool
    r"""Indicates whether this transaction has been settled."""
    intent: TransactionIntent
    gift_card_redemptions: List[GiftCardRedemptionTypedDict]
    r"""The gift cards redeemed for this transaction."""
    created_at: datetime
    r"""The date and time when the transaction was created, in ISO 8601 format."""
    updated_at: datetime
    r"""The date and time when the transaction was last updated, in ISO 8601 format."""
    disputed: bool
    r"""Indicates whether this transaction has been disputed."""
    payment_source: TransactionPaymentSource
    r"""The way payment method information made it to this transaction."""
    merchant_initiated: bool
    r"""Indicates whether the transaction was initiated by the merchant or the customer."""
    is_subsequent_payment: bool
    r"""Indicates whether the transaction represents a subsequent payment or an initial one."""
    intent_outcome: TransactionIntentOutcome
    multi_tender: bool
    r"""The outcome of the original intent of a transaction. This allows you to understand if the intent of the transaction (e.g. `capture` or `authorize`) has been achieved when dealing with multiple payment instruments."""
    account_funding_transaction: bool
    r"""Marks the transaction as an AFT. Requires the payment service to support this feature, and might `recipient` and `buyer` data"""
    type: Literal["transaction"]
    r"""Always `transaction`."""
    settled_currency: NotRequired[Nullable[str]]
    r"""The ISO 4217 currency code of this transaction's settlement."""
    country: NotRequired[Nullable[str]]
    r"""The 2-letter ISO 3166-1 alpha-2 country code for the transaction. Used to filter payment services for processing."""
    external_identifier: NotRequired[Nullable[str]]
    r"""An external identifier that can be used to match the transaction against your own records."""
    payment_method: NotRequired[Nullable[TransactionPaymentMethodTypedDict]]
    r"""The payment method used for this transaction."""
    method: NotRequired[Nullable[Method]]
    r"""The method used for the transaction."""
    instrument_type: NotRequired[Nullable[InstrumentType]]
    r"""The name of the instrument used to process the transaction."""
    error_code: NotRequired[Nullable[str]]
    r"""The standardized error code set by Gr4vy."""
    payment_service: NotRequired[Nullable[TransactionPaymentServiceTypedDict]]
    r"""The payment service used for this transaction."""
    pending_review: NotRequired[bool]
    r"""Whether a manual anti fraud review is pending with an anti fraud service."""
    buyer: NotRequired[Nullable[TransactionBuyerTypedDict]]
    r"""The buyer used for this transaction."""
    raw_response_code: NotRequired[Nullable[str]]
    r"""This is the response code received from the payment service. This can be set to any value and is not standardized across different payment services."""
    raw_response_description: NotRequired[Nullable[str]]
    r"""This is the response description received from the payment service. This can be set to any value and is not standardized across different payment services."""
    shipping_details: NotRequired[Nullable[ShippingDetailsTypedDict]]
    r"""The shipping details associated with the transaction."""
    checkout_session_id: NotRequired[Nullable[str]]
    r"""The identifier for the checkout session this transaction is associated with."""
    gift_card_service: NotRequired[Nullable[GiftCardServiceTypedDict]]
    r"""The gift card service used for this transaction."""
    airline: NotRequired[Nullable[AirlineTypedDict]]
    r"""Contains information about an airline travel, if applicable."""
    auth_response_code: NotRequired[Nullable[str]]
    r"""This is the response description received from the processor."""
    avs_response_code: NotRequired[Nullable[AVSResponseCode]]
    r"""The response code received from the payment service for the Address Verification Check (AVS). This code is mapped to a standardized Gr4vy AVS response code."""
    cvv_response_code: NotRequired[Nullable[CVVResponseCode]]
    r"""The response code received from the payment service for the Card Verification Value (CVV). This code is mapped to a standardized Gr4vy CVV response code."""
    anti_fraud_decision: NotRequired[Nullable[AntiFraudDecision]]
    r"""The mapped decision received from the anti-fraud service. In case of a review decision this field is not updated once the review is resolved."""
    cart_items: NotRequired[Nullable[List[CartItemTypedDict]]]
    r"""An array of cart items that represents the line items of a transaction."""
    statement_descriptor: NotRequired[Nullable[StatementDescriptorTypedDict]]
    r"""The statement descriptor is the text to be shown on the buyer's statements."""
    scheme_transaction_id: NotRequired[Nullable[str]]
    r"""An identifier for the transaction used by the scheme itself, when available."""
    three_d_secure: NotRequired[Nullable[TransactionThreeDSecureSummaryTypedDict]]
    r"""The 3-D Secure data that was sent to the payment service for the transaction."""
    payment_service_transaction_id: NotRequired[Nullable[str]]
    r"""The payment service's unique ID for the transaction."""
    additional_identifiers: NotRequired[Dict[str, Nullable[str]]]
    r"""A list of additional identifiers that we may keep track of to manage this transaction. This may include the authorization ID, capture ID, and processor ID, as well as an undefined list of additional identifiers."""
    metadata: NotRequired[Nullable[Dict[str, str]]]
    r"""Additional information about the transaction stored as key-value pairs."""
    authorized_at: NotRequired[Nullable[datetime]]
    r"""The date this transaction was authorized at."""
    captured_at: NotRequired[Nullable[datetime]]
    r"""The date this transaction was captured at."""
    voided_at: NotRequired[Nullable[datetime]]
    r"""The date this transaction was voided at."""
    canceled_at: NotRequired[Nullable[datetime]]
    r"""The date this transaction was canceled at."""
    approval_expires_at: NotRequired[Nullable[datetime]]
    r"""The date this transaction's approval URL will expire at."""
    buyer_approval_timedout_at: NotRequired[Nullable[datetime]]
    r"""The date this transaction's approval timed out at."""
    recipient: NotRequired[Nullable[RecipientTypedDict]]
    r"""The recipient of any account to account funding. For use with AFTs."""
    merchant_advice_code: NotRequired[Nullable[str]]
    r"""An optional merchant advice code which provides insight into the type of transaction or reason why the payment failed."""
    installment_count: NotRequired[Nullable[int]]
    r"""The number of installments for this transaction, if applicable."""
    tax_amount: NotRequired[Nullable[int]]
    r"""The sales tax amount for this transaction, represented as a monetary amount in the smallest currency unit for the given currency, for example `1299` cents to create an authorization for `$12.99`"""
    merchant_tax_id: NotRequired[Nullable[str]]
    r"""Merchant tax ID (for example, EIN or VAT number)."""
    customer_reference_number: NotRequired[Nullable[str]]
    r"""Customer code or reference."""
    amount_includes_tax: NotRequired[Nullable[bool]]
    r"""Whether the tax is included in the amount."""
    supplier_order_number: NotRequired[Nullable[str]]
    r"""The merchant's unique identifier for the sales order or invoice."""
    duty_amount: NotRequired[Nullable[int]]
    r"""Total charges for import/export duties."""
    shipping_amount: NotRequired[Nullable[int]]
    r"""Total shipping amount."""


class Transaction(BaseModel):
    r"""A full transaction resource."""

    id: str
    r"""The ID for the transaction."""

    reconciliation_id: str
    r"""The base62 encoded transaction ID. This represents a shorter version of this transaction's `id` which is sent to payment services, anti-fraud services, and other connectors. You can use this ID to reconcile a payment service's transaction against our system. This ID is sent instead of the transaction ID because not all services support 36 digit identifiers."""

    merchant_account_id: str
    r"""The ID of the merchant account this transaction belongs to."""

    currency: str
    r"""The currency code for this transaction."""

    amount: int
    r"""The total amount for this transaction across all funding sources including gift cards."""

    status: Annotated[TransactionStatus, PlainValidator(validate_open_enum(False))]

    authorized_amount: int
    r"""The amount for this transaction that has been authorized for the `payment_method`. This can be less than the `amount` if gift cards were used."""

    captured_amount: int
    r"""The total amount captured for this transaction, in the smallest currency unit (for example, cents or pence). This can be the full value of the `authorized_amount` or less."""

    refunded_amount: int
    r"""The total amount refunded for this transaction, in the smallest currency unit (for example, cents or pence). This can be the full value of the `captured_amount` or less."""

    settled_amount: int
    r"""The net amount settled for this transaction, in the smallest currency unit (for example, cents or pence)."""

    settled: bool
    r"""Indicates whether this transaction has been settled."""

    intent: Annotated[TransactionIntent, PlainValidator(validate_open_enum(False))]

    gift_card_redemptions: List[GiftCardRedemption]
    r"""The gift cards redeemed for this transaction."""

    created_at: datetime
    r"""The date and time when the transaction was created, in ISO 8601 format."""

    updated_at: datetime
    r"""The date and time when the transaction was last updated, in ISO 8601 format."""

    disputed: bool
    r"""Indicates whether this transaction has been disputed."""

    payment_source: Annotated[
        TransactionPaymentSource, PlainValidator(validate_open_enum(False))
    ]
    r"""The way payment method information made it to this transaction."""

    merchant_initiated: bool
    r"""Indicates whether the transaction was initiated by the merchant or the customer."""

    is_subsequent_payment: bool
    r"""Indicates whether the transaction represents a subsequent payment or an initial one."""

    intent_outcome: Annotated[
        TransactionIntentOutcome, PlainValidator(validate_open_enum(False))
    ]

    multi_tender: bool
    r"""The outcome of the original intent of a transaction. This allows you to understand if the intent of the transaction (e.g. `capture` or `authorize`) has been achieved when dealing with multiple payment instruments."""

    account_funding_transaction: bool
    r"""Marks the transaction as an AFT. Requires the payment service to support this feature, and might `recipient` and `buyer` data"""

    TYPE: Annotated[
        Annotated[
            Optional[Literal["transaction"]],
            AfterValidator(validate_const("transaction")),
        ],
        pydantic.Field(alias="type"),
    ] = "transaction"
    r"""Always `transaction`."""

    settled_currency: OptionalNullable[str] = UNSET
    r"""The ISO 4217 currency code of this transaction's settlement."""

    country: OptionalNullable[str] = UNSET
    r"""The 2-letter ISO 3166-1 alpha-2 country code for the transaction. Used to filter payment services for processing."""

    external_identifier: OptionalNullable[str] = UNSET
    r"""An external identifier that can be used to match the transaction against your own records."""

    payment_method: OptionalNullable[TransactionPaymentMethod] = UNSET
    r"""The payment method used for this transaction."""

    method: Annotated[
        OptionalNullable[Method], PlainValidator(validate_open_enum(False))
    ] = UNSET
    r"""The method used for the transaction."""

    instrument_type: Annotated[
        OptionalNullable[InstrumentType], PlainValidator(validate_open_enum(False))
    ] = UNSET
    r"""The name of the instrument used to process the transaction."""

    error_code: OptionalNullable[str] = UNSET
    r"""The standardized error code set by Gr4vy."""

    payment_service: OptionalNullable[TransactionPaymentService] = UNSET
    r"""The payment service used for this transaction."""

    pending_review: Optional[bool] = False
    r"""Whether a manual anti fraud review is pending with an anti fraud service."""

    buyer: OptionalNullable[TransactionBuyer] = UNSET
    r"""The buyer used for this transaction."""

    raw_response_code: OptionalNullable[str] = UNSET
    r"""This is the response code received from the payment service. This can be set to any value and is not standardized across different payment services."""

    raw_response_description: OptionalNullable[str] = UNSET
    r"""This is the response description received from the payment service. This can be set to any value and is not standardized across different payment services."""

    shipping_details: OptionalNullable[ShippingDetails] = UNSET
    r"""The shipping details associated with the transaction."""

    checkout_session_id: OptionalNullable[str] = UNSET
    r"""The identifier for the checkout session this transaction is associated with."""

    gift_card_service: OptionalNullable[GiftCardService] = UNSET
    r"""The gift card service used for this transaction."""

    airline: OptionalNullable[Airline] = UNSET
    r"""Contains information about an airline travel, if applicable."""

    auth_response_code: OptionalNullable[str] = UNSET
    r"""This is the response description received from the processor."""

    avs_response_code: Annotated[
        OptionalNullable[AVSResponseCode], PlainValidator(validate_open_enum(False))
    ] = UNSET
    r"""The response code received from the payment service for the Address Verification Check (AVS). This code is mapped to a standardized Gr4vy AVS response code."""

    cvv_response_code: Annotated[
        OptionalNullable[CVVResponseCode], PlainValidator(validate_open_enum(False))
    ] = UNSET
    r"""The response code received from the payment service for the Card Verification Value (CVV). This code is mapped to a standardized Gr4vy CVV response code."""

    anti_fraud_decision: Annotated[
        OptionalNullable[AntiFraudDecision], PlainValidator(validate_open_enum(False))
    ] = UNSET
    r"""The mapped decision received from the anti-fraud service. In case of a review decision this field is not updated once the review is resolved."""

    cart_items: OptionalNullable[List[CartItem]] = UNSET
    r"""An array of cart items that represents the line items of a transaction."""

    statement_descriptor: OptionalNullable[StatementDescriptor] = UNSET
    r"""The statement descriptor is the text to be shown on the buyer's statements."""

    scheme_transaction_id: OptionalNullable[str] = UNSET
    r"""An identifier for the transaction used by the scheme itself, when available."""

    three_d_secure: OptionalNullable[TransactionThreeDSecureSummary] = UNSET
    r"""The 3-D Secure data that was sent to the payment service for the transaction."""

    payment_service_transaction_id: OptionalNullable[str] = UNSET
    r"""The payment service's unique ID for the transaction."""

    additional_identifiers: Optional[Dict[str, Nullable[str]]] = None
    r"""A list of additional identifiers that we may keep track of to manage this transaction. This may include the authorization ID, capture ID, and processor ID, as well as an undefined list of additional identifiers."""

    metadata: OptionalNullable[Dict[str, str]] = UNSET
    r"""Additional information about the transaction stored as key-value pairs."""

    authorized_at: OptionalNullable[datetime] = UNSET
    r"""The date this transaction was authorized at."""

    captured_at: OptionalNullable[datetime] = UNSET
    r"""The date this transaction was captured at."""

    voided_at: OptionalNullable[datetime] = UNSET
    r"""The date this transaction was voided at."""

    canceled_at: OptionalNullable[datetime] = UNSET
    r"""The date this transaction was canceled at."""

    approval_expires_at: OptionalNullable[datetime] = UNSET
    r"""The date this transaction's approval URL will expire at."""

    buyer_approval_timedout_at: OptionalNullable[datetime] = UNSET
    r"""The date this transaction's approval timed out at."""

    recipient: OptionalNullable[Recipient] = UNSET
    r"""The recipient of any account to account funding. For use with AFTs."""

    merchant_advice_code: OptionalNullable[str] = UNSET
    r"""An optional merchant advice code which provides insight into the type of transaction or reason why the payment failed."""

    installment_count: OptionalNullable[int] = UNSET
    r"""The number of installments for this transaction, if applicable."""

    tax_amount: OptionalNullable[int] = UNSET
    r"""The sales tax amount for this transaction, represented as a monetary amount in the smallest currency unit for the given currency, for example `1299` cents to create an authorization for `$12.99`"""

    merchant_tax_id: OptionalNullable[str] = UNSET
    r"""Merchant tax ID (for example, EIN or VAT number)."""

    customer_reference_number: OptionalNullable[str] = UNSET
    r"""Customer code or reference."""

    amount_includes_tax: OptionalNullable[bool] = UNSET
    r"""Whether the tax is included in the amount."""

    supplier_order_number: OptionalNullable[str] = UNSET
    r"""The merchant's unique identifier for the sales order or invoice."""

    duty_amount: OptionalNullable[int] = UNSET
    r"""Total charges for import/export duties."""

    shipping_amount: OptionalNullable[int] = UNSET
    r"""Total shipping amount."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "type",
            "settled_currency",
            "country",
            "external_identifier",
            "payment_method",
            "method",
            "instrument_type",
            "error_code",
            "payment_service",
            "pending_review",
            "buyer",
            "raw_response_code",
            "raw_response_description",
            "shipping_details",
            "checkout_session_id",
            "gift_card_service",
            "airline",
            "auth_response_code",
            "avs_response_code",
            "cvv_response_code",
            "anti_fraud_decision",
            "cart_items",
            "statement_descriptor",
            "scheme_transaction_id",
            "three_d_secure",
            "payment_service_transaction_id",
            "additional_identifiers",
            "metadata",
            "authorized_at",
            "captured_at",
            "voided_at",
            "canceled_at",
            "approval_expires_at",
            "buyer_approval_timedout_at",
            "recipient",
            "merchant_advice_code",
            "installment_count",
            "tax_amount",
            "merchant_tax_id",
            "customer_reference_number",
            "amount_includes_tax",
            "supplier_order_number",
            "duty_amount",
            "shipping_amount",
        ]
        nullable_fields = [
            "settled_currency",
            "country",
            "external_identifier",
            "payment_method",
            "method",
            "instrument_type",
            "error_code",
            "payment_service",
            "buyer",
            "raw_response_code",
            "raw_response_description",
            "shipping_details",
            "checkout_session_id",
            "gift_card_service",
            "airline",
            "auth_response_code",
            "avs_response_code",
            "cvv_response_code",
            "anti_fraud_decision",
            "cart_items",
            "statement_descriptor",
            "scheme_transaction_id",
            "three_d_secure",
            "payment_service_transaction_id",
            "metadata",
            "authorized_at",
            "captured_at",
            "voided_at",
            "canceled_at",
            "approval_expires_at",
            "buyer_approval_timedout_at",
            "recipient",
            "merchant_advice_code",
            "installment_count",
            "tax_amount",
            "merchant_tax_id",
            "customer_reference_number",
            "amount_includes_tax",
            "supplier_order_number",
            "duty_amount",
            "shipping_amount",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from gr4vy.types import UnrecognizedStr
from typing import Literal, Union


TaxIDKind = Union[
    Literal[
        "ae.trn",
        "au.abn",
        "ar.dni",
        "ar.cuil",
        "ar.cuit",
        "br.cnpj",
        "br.cpf",
        "ca.bn",
        "ca.gst_hst",
        "ca.pst_bc",
        "ca.pst_mb",
        "ca.pst_sk",
        "ca.qst",
        "ch.vat",
        "cl.tin",
        "co.itin",
        "co.nit",
        "es.cif",
        "eu.vat",
        "gb.vat",
        "hk.br",
        "id.nik",
        "id.npwp",
        "in.gst",
        "in.pan",
        "jp.cn",
        "jp.rn",
        "kr.brn",
        "li.uid",
        "mx.curp",
        "mx.rfc",
        "my.frp",
        "my.itn",
        "my.nric",
        "my.sst",
        "no.vat",
        "nz.gst",
        "pe.ruc",
        "ph.tin",
        "ru.inn",
        "ru.kpp",
        "sa.vat",
        "sg.gst",
        "sg.uen",
        "th.id",
        "th.vat",
        "tw.vat",
        "us.ein",
        "za.vat",
        "bo.ci",
        "uy.rut",
        "uy.ci",
    ],
    UnrecognizedStr,
]



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .forterantifraudoptionscartitembeneficiaryaddress import (
    ForterAntiFraudOptionsCartItemBeneficiaryAddress,
    ForterAntiFraudOptionsCartItemBeneficiaryAddressTypedDict,
)
from .forterantifraudoptionscartitembeneficiarycomments import (
    ForterAntiFraudOptionsCartItemBeneficiaryComments,
    ForterAntiFraudOptionsCartItemBeneficiaryCommentsTypedDict,
)
from .forterantifraudoptionscartitembeneficiarypersonaldetails import (
    ForterAntiFraudOptionsCartItemBeneficiaryPersonalDetails,
    ForterAntiFraudOptionsCartItemBeneficiaryPersonalDetailsTypedDict,
)
from .forterantifraudoptionscartitembeneficiaryphone import (
    ForterAntiFraudOptionsCartItemBeneficiaryPhone,
    ForterAntiFraudOptionsCartItemBeneficiaryPhoneTypedDict,
)
from gr4vy.types import BaseModel, Nullable, OptionalNullable, UNSET, UNSET_SENTINEL
from pydantic import model_serializer
from typing import List
from typing_extensions import NotRequired, TypedDict


class ForterAntiFraudOptionsCartItemBeneficiaryTypedDict(TypedDict):
    personal_details: ForterAntiFraudOptionsCartItemBeneficiaryPersonalDetailsTypedDict
    address: NotRequired[
        Nullable[ForterAntiFraudOptionsCartItemBeneficiaryAddressTypedDict]
    ]
    r"""Address information of the beneficiary."""
    phone: NotRequired[
        Nullable[List[ForterAntiFraudOptionsCartItemBeneficiaryPhoneTypedDict]]
    ]
    r"""Phone numbers associated with the beneficiary."""
    comments: NotRequired[
        Nullable[ForterAntiFraudOptionsCartItemBeneficiaryCommentsTypedDict]
    ]
    r"""Comments related to the beneficiary."""


class ForterAntiFraudOptionsCartItemBeneficiary(BaseModel):
    personal_details: ForterAntiFraudOptionsCartItemBeneficiaryPersonalDetails

    address: OptionalNullable[ForterAntiFraudOptionsCartItemBeneficiaryAddress] = UNSET
    r"""Address information of the beneficiary."""

    phone: OptionalNullable[List[ForterAntiFraudOptionsCartItemBeneficiaryPhone]] = (
        UNSET
    )
    r"""Phone numbers associated with the beneficiary."""

    comments: OptionalNullable[ForterAntiFraudOptionsCartItemBeneficiaryComments] = (
        UNSET
    )
    r"""Comments related to the beneficiary."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["address", "phone", "comments"]
        nullable_fields = ["address", "phone", "comments"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from gr4vy.types import BaseModel
from typing_extensions import TypedDict


class ClickToPaySessionRequestTypedDict(TypedDict):
    checkout_session_id: str
    r"""The checkout session ID to create a Click to Pay session for."""


class ClickToPaySessionRequest(BaseModel):
    checkout_session_id: str
    r"""The checkout session ID to create a Click to Pay session for."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from gr4vy.types import UnrecognizedStr
from typing import Literal, Union


ReportSpecModel = Union[
    Literal[
        "transactions",
        "transaction_retries",
        "detailed_settlement",
        "accounts_receivables",
    ],
    UnrecognizedStr,
]



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .transactionconnectionoptions import (
    TransactionConnectionOptions,
    TransactionConnectionOptionsTypedDict,
)
from gr4vy.types import BaseModel, Nullable, OptionalNullable, UNSET, UNSET_SENTINEL
from pydantic import model_serializer
from typing import Dict
from typing_extensions import NotRequired, TypedDict


class TransactionUpdateTypedDict(TypedDict):
    external_identifier: NotRequired[Nullable[str]]
    r"""An external identifier that can be used to match the transaction against your own records."""
    metadata: NotRequired[Nullable[Dict[str, str]]]
    r"""Additional information about the transaction stored as key-value pairs. If provided, the whole value will be overridden."""
    connection_options: NotRequired[Nullable[TransactionConnectionOptionsTypedDict]]
    r"""Allows for passing optional configuration per connection to take advantage of connection specific features. When provided, the data is only passed to the target connection type to prevent sharing configuration across connections. Please note that each of the keys this object are in kebab-case, for example `cybersource-anti-fraud` as they represent the ID of the connector. All the other keys will be snake case, for example `merchant_defined_data` or camel case to match an external API that the connector uses. If provided, the whole value will be overridden."""


class TransactionUpdate(BaseModel):
    external_identifier: OptionalNullable[str] = UNSET
    r"""An external identifier that can be used to match the transaction against your own records."""

    metadata: OptionalNullable[Dict[str, str]] = UNSET
    r"""Additional information about the transaction stored as key-value pairs. If provided, the whole value will be overridden."""

    connection_options: OptionalNullable[TransactionConnectionOptions] = UNSET
    r"""Allows for passing optional configuration per connection to take advantage of connection specific features. When provided, the data is only passed to the target connection type to prevent sharing configuration across connections. Please note that each of the keys this object are in kebab-case, for example `cybersource-anti-fraud` as they represent the ID of the connector. All the other keys will be snake case, for example `merchant_defined_data` or camel case to match an external API that the connector uses. If provided, the whole value will be overridden."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["external_identifier", "metadata", "connection_options"]
        nullable_fields = ["external_identifier", "metadata", "connection_options"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from gr4vy.types import BaseModel, Nullable, OptionalNullable, UNSET, UNSET_SENTINEL
from pydantic import model_serializer
from typing_extensions import NotRequired, TypedDict


class ThreeDSecureV2TypedDict(TypedDict):
    version: str
    authentication_response: NotRequired[Nullable[str]]
    directory_response: NotRequired[Nullable[str]]
    directory_transaction_id: NotRequired[Nullable[str]]
    transaction_reason: NotRequired[Nullable[str]]
    cavv: NotRequired[Nullable[str]]
    eci: NotRequired[Nullable[str]]
    cardholder_info: NotRequired[Nullable[str]]


class ThreeDSecureV2(BaseModel):
    version: str

    authentication_response: OptionalNullable[str] = UNSET

    directory_response: OptionalNullable[str] = UNSET

    directory_transaction_id: OptionalNullable[str] = UNSET

    transaction_reason: OptionalNullable[str] = UNSET

    cavv: OptionalNullable[str] = UNSET

    eci: OptionalNullable[str] = UNSET

    cardholder_info: OptionalNullable[str] = UNSET

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "authentication_response",
            "directory_response",
            "directory_transaction_id",
            "transaction_reason",
            "cavv",
            "eci",
            "cardholder_info",
        ]
        nullable_fields = [
            "authentication_response",
            "directory_response",
            "directory_transaction_id",
            "transaction_reason",
            "cavv",
            "eci",
            "cardholder_info",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m
